@@ -32,6 +32,7 @@ from breezy.workspace import reset_tree, check_clean_tree
 from ..apply import (
     ResultFileFormatError,
     ScriptFailed,
+    ScriptNotFound,
     ScriptMadeNoChanges,
     )
 
@@ -54,11 +55,13 @@ class MissingChangelog(Exception):
 class DetailedFailure(Exception):
     """Detailed failure"""
 
-    def __init__(self, source_name, result_code, description, details=None):
+    def __init__(self, source_name, result_code, description, stage=None,
+                 details=None):
         self.source = source_name
         self.result_code = result_code
         self.description = description
         self.details = details
+        self.stage = stage
 
     @classmethod
     def from_json(cls, source_name, json):
@@ -66,6 +69,7 @@ class DetailedFailure(Exception):
             source_name,
             result_code=json.get('result_code'),
             description=json.get('description'),
+            stage=tuple(json['stage']) if json.get('stage') else None,
             details=json.get('details'))
 
 
@@ -187,9 +191,13 @@ def script_runner(   # noqa: C901
             env['SVP_RESUME'] = os.path.join(td, 'resume-metadata.json')
             with open(env['SVP_RESUME'], 'w') as f:
                 json.dump(resume_metadata, f)
-        p = subprocess.Popen(
-            script, cwd=local_tree.abspath(subpath), stdout=subprocess.PIPE,
-            shell=isinstance(script, str), env=env)
+        try:
+            p = subprocess.Popen(
+                script, cwd=local_tree.abspath(subpath),
+                stdout=subprocess.PIPE,
+                shell=isinstance(script, str), env=env)
+        except FileNotFoundError as e:
+            raise ScriptNotFound(script) from e
         (description_encoded, err) = p.communicate(b"")
         try:
             with open(env['SVP_RESULT'], 'r') as f:
@@ -332,8 +340,7 @@ def main(argv: List[str]) -> Optional[int]:  # noqa: C901
     elif recipe and recipe.command:
         command = recipe.command
     else:
-        logging.exception('No command or recipe specified.')
-        return 1
+        parser.error('No command or recipe specified.')
 
     local_tree, subpath = WorkingTree.open_containing('.')
 
