@@ -26,26 +26,22 @@ from breezy import (
 )
 from breezy.errors import PermissionDenied
 from breezy.memorybranch import MemoryBranch
-try:
-    from breezy.forge import (
-        get_forge,
-        Forge,
-        MergeProposal,
-        MergeProposalExists,
-        NoSuchProject,
-        UnsupportedForge,
-        SourceNotDerivedFromTarget,
-        )
-except ModuleNotFoundError:
-    from breezy.propose import (
-        get_hoster as get_forge,
-        Hoster as Forge,
-        MergeProposal,
-        MergeProposalExists,
-        NoSuchProject,
-        UnsupportedHoster as UnsupportedForge,
-        SourceNotDerivedFromTarget,
+from breezy.forge import (
+    get_forge,
+    Forge,
+    MergeProposal,
+    MergeProposalExists,
+    NoSuchProject,
+    UnsupportedForge,
+    SourceNotDerivedFromTarget,
     )
+try:
+    from breezy.forge import TitleUnsupported
+except ImportError:  # breezy < 3.3.1
+    from breezy.errors import (  # type: ignore
+        UnsupportedOperation as TitleUnsupported
+    )
+
 from breezy.transport import Transport
 
 from .utils import (
@@ -82,8 +78,10 @@ SUPPORTED_MODES: List[str] = [
 
 
 def _tag_selector_from_tags(tags):
-    # TODO(jelmer): Select dict
-    return tags.__contains__
+    def select(tag):
+        assert isinstance(tag, str)
+        return tags.__contains__(tag)
+    return select
 
 
 def push_result(
@@ -182,12 +180,14 @@ def propose_changes(  # noqa: C901
     forge: Forge,
     name: str,
     mp_description: str,
+    *,
     resume_branch: Optional[Branch] = None,
     resume_proposal: Optional[MergeProposal] = None,
     overwrite_existing: Optional[bool] = True,
     labels: Optional[List[str]] = None,
     dry_run: bool = False,
     commit_message: Optional[str] = None,
+    title: Optional[str] = None,
     additional_colocated_branches:
         Optional[Union[List[str], Dict[str, str]]] = None,
     allow_empty: bool = False,
@@ -210,6 +210,7 @@ def propose_changes(  # noqa: C901
       labels: Labels to add
       dry_run: Whether to just dry-run the change
       commit_message: Optional commit message
+      title: Optional title
       additional_colocated_branches: Additional colocated branches to propose
       allow_empty: Whether to allow empty merge proposals
       reviewers: List of reviewers
@@ -269,7 +270,6 @@ def propose_changes(  # noqa: C901
         )
     if (
         resume_proposal is not None
-        and getattr(resume_proposal, "is_closed", None)
         and resume_proposal.is_closed()
     ):
         from breezy.propose import (
@@ -286,7 +286,10 @@ def propose_changes(  # noqa: C901
         if not dry_run:
             proposal_builder = forge.get_proposer(remote_branch, main_branch)
             kwargs: Dict[str, Any] = {}
-            kwargs["commit_message"] = commit_message
+            if forge.supports_merge_proposal_commit_message:
+                kwargs["commit_message"] = commit_message
+            if getattr(forge, "supports_merge_proposal_title", False):
+                kwargs["title"] = title
             kwargs["allow_collaboration"] = allow_collaboration
             try:
                 mp = proposal_builder.create_proposal(
@@ -296,7 +299,7 @@ def propose_changes(  # noqa: C901
                     **kwargs
                 )
             except MergeProposalExists as e:
-                if getattr(e, "existing_proposal", None) is None:
+                if e.existing_proposal is None:
                     # Forge didn't tell us where the actual proposal is.
                     raise
                 resume_proposal = e.existing_proposal
@@ -313,6 +316,7 @@ def propose_changes(  # noqa: C901
                 labels=labels,
                 description=mp_description,
                 commit_message=commit_message,
+                title=title,
                 reviewers=reviewers,
                 owner=owner,
                 stop_revision=stop_revision,
@@ -381,6 +385,7 @@ class DryRunProposal(MergeProposal):
         labels: Optional[List[str]] = None,
         description: Optional[str] = None,
         commit_message: Optional[str] = None,
+        title: Optional[str] = None,
         reviewers: Optional[List[str]] = None,
         owner: Optional[str] = None,
         stop_revision: Optional[bytes] = None,
@@ -391,6 +396,7 @@ class DryRunProposal(MergeProposal):
         self.source_branch = source_branch
         self.target_branch = target_branch
         self.commit_message = commit_message
+        self.title = title
         self.url = None
         self.reviewers = reviewers
         self.owner = owner
@@ -403,11 +409,16 @@ class DryRunProposal(MergeProposal):
         if source_branch is None:
             source_branch = open_branch(mp.get_source_branch_url())
         commit_message = mp.get_commit_message()
+        try:
+            title = mp.get_title()
+        except TitleUnsupported:
+            title = None
         return cls(
             source_branch=source_branch,
             target_branch=open_branch(mp.get_target_branch_url()),
             description=mp.get_description(),
             commit_message=commit_message,
+            title=title,
         )
 
     def __repr__(self) -> str:
@@ -430,6 +441,12 @@ class DryRunProposal(MergeProposal):
     def set_commit_message(self, commit_message: str) -> None:
         self.commit_message = commit_message
 
+    def get_title(self) -> Optional[str]:
+        return self.title
+
+    def set_title(self, title: str) -> None:
+        self.title = title
+
     def get_source_branch_url(self) -> str:
         """Return the source branch."""
         return full_branch_url(self.source_branch)
@@ -602,10 +619,13 @@ def publish_changes(
     resume_branch: Optional[Branch],
     mode: str,
     name: str,
+    *,
     get_proposal_description: Callable[[str, Optional[MergeProposal]], str],
     get_proposal_commit_message: Callable[
         [Optional[MergeProposal]], Optional[str]
     ] = None,
+    get_proposal_title:
+        Callable[[Optional[MergeProposal]], Optional[str]] = None,
     dry_run: bool = False,
     forge: Optional[Forge] = None,
     allow_create_proposal: bool = True,
@@ -626,6 +646,7 @@ def publish_changes(
       name: Branch name to push
       get_proposal_description: Function to retrieve proposal description
       get_proposal_commit_message: Function to retrieve proposal commit message
+      get_proposal_title: Function to retrieve proposal title
       dry_run: Whether to dry run
       forge: Forge, if known
       allow_create_proposal: Whether to allow creating proposals
@@ -711,6 +732,12 @@ def publish_changes(
         commit_message = get_proposal_commit_message(
             existing_proposal if resume_branch else None
         )
+    if get_proposal_title is not None:
+        title = get_proposal_title(
+            existing_proposal if resume_branch else None
+        )
+    else:
+        title = None
     (proposal, is_new) = propose_changes(
         local_branch,
         main_branch,
@@ -723,6 +750,7 @@ def publish_changes(
         labels=labels,
         dry_run=dry_run,
         commit_message=commit_message,
+        title=title,
         reviewers=reviewers,
         tags=tags,
         owner=derived_owner,
