@@ -17,6 +17,7 @@
 
 from dataclasses import dataclass
 from jinja2 import Template
+import os
 from typing import Optional, Dict, Union, List
 import yaml
 
@@ -29,9 +30,45 @@ class Recipe(object):
     command: Union[str, List[str]]
     merge_request_description_template: Dict[Optional[str], Template]
     merge_request_commit_message_template: Template
+    merge_request_title_template: Template
     resume: bool = False
     commit_pending: Optional[bool] = True
     propose_threshold: Optional[int] = None
+    mode: Optional[str] = None
+    labels: Optional[List[str]] = None
+
+    def to_yaml(self):
+        ret = {}
+        if self.mode:
+            ret['mode'] = self.mode
+        if self.labels:
+            ret['labels'] = self.labels
+        if (self.merge_request_description_template
+                or self.merge_request_commit_message_template
+                or self.merge_request_title_template
+                or self.propose_threshold is not None):
+            ret['merge-request'] = {}
+            if self.merge_request_description_template:
+                ret['merge-request']['description'] = (
+                    self.merge_request_description_template)
+            if self.merge_request_commit_message_template:
+                ret['merge-request']['commit-message'] = (
+                    self.merge_request_commit_message_template)
+            if self.merge_request_title_template:
+                ret['merge-request']['title'] = (
+                    self.merge_request_title_template)
+            if self.propose_threshold is not None:
+                ret['merge-request']['propose-threshold'] = (
+                    self.propose_threshold)
+        if self.command:
+            ret['command'] = self.command
+        if self.name:
+            ret['name'] = self.name
+        if self.commit_pending is not None:
+            ret['commit-pending'] = self.commit_pending
+        if self.resume is not None:
+            ret['resume'] = self.resume
+        return ret
 
     @classmethod
     def from_yaml(cls, d):
@@ -44,20 +81,27 @@ class Recipe(object):
                 merge_request_description_template = {None: description}
             merge_request_commit_message_template = merge_request.get(
                 'commit-message')
+            merge_request_title_template = merge_request.get(
+                'title')
             propose_threshold = merge_request.get('propose-threshold')
         else:
             merge_request_description_template = {}
             merge_request_commit_message_template = None
+            merge_request_title_template = None
             propose_threshold = None
         return cls(
             name=d.get('name'),
+            labels=d.get('labels', []),
             command=d.get('command'),
+            mode=d.get('mode'),
             resume=d.get('resume', False),
             commit_pending=d.get('commit-pending'),
             merge_request_description_template=(
                 merge_request_description_template),
             merge_request_commit_message_template=(
                 merge_request_commit_message_template),
+            merge_request_title_template=(
+                merge_request_title_template),
             propose_threshold=propose_threshold)
 
     def render_merge_request_commit_message(self, context):
@@ -79,4 +123,7 @@ class Recipe(object):
     @classmethod
     def from_path(cls, path):
         with open(path, 'r') as f:
-            return cls.from_yaml(yaml.full_load(f))
+            ret = cls.from_yaml(yaml.full_load(f))
+            if not ret.name:
+                ret.name = os.path.splitext(os.path.basename(path))[0]
+            return ret
